package com.jiuqi.util.patterns;

import java.util.Map;
import java.util.ArrayList;
import java.util.List;

class PlaceholderPattern implements IPattern
{
    private List items;
    private String expression;
    private boolean isVariable;
    
    public PlaceholderPattern() {
        this.items = new ArrayList();
    }
    
    @Override
    public String toString() {
        final StringBuffer buffer = new StringBuffer();
        if (this.isVariable) {
            buffer.append("${");
        }
        for (int i = 0; i < this.size(); ++i) {
            buffer.append(this.get(i).toString());
        }
        if (this.isVariable) {
            buffer.append("}");
        }
        return buffer.toString();
    }
    
    public boolean isVariable() {
        return this.isVariable;
    }
    
    public void setIsVariable(final boolean value) {
        this.isVariable = value;
    }
    
    public void parse(final String expression) throws PatternException {
        this.clear();
        this.expression = expression;
        if (expression == null) {
            return;
        }
        int cur = 0;
        int start = 0;
        int stackSize = 0;
        boolean startVar = false;
        while (cur < expression.length()) {
            switch (this.charOf(cur)) {
                case '$': {
                    if (this.charOf(cur + 1) == '{') {
                        if (startVar) {
                            ++stackSize;
                        }
                        else {
                            this.addConst(start, cur);
                            start = cur;
                            startVar = true;
                            stackSize = 0;
                        }
                        cur += 2;
                        continue;
                    }
                    ++cur;
                    continue;
                }
                case '}': {
                    if (startVar) {
                        if (stackSize == 0) {
                            this.addVariable(start + 2, cur);
                            startVar = false;
                            start = cur + 1;
                        }
                        else {
                            --stackSize;
                        }
                    }
                    ++cur;
                    continue;
                }
                default: {
                    ++cur;
                    continue;
                }
            }
        }
        if (cur == expression.length() && cur > start) {
            this.addConst(start, cur);
        }
    }
    
    private char charOf(final int index) {
        return (index < 0 || index >= this.expression.length()) ? '\0' : this.expression.charAt(index);
    }
    
    private void addConst(final int start, final int cur) {
        if (start >= cur) {
            return;
        }
        final String s = this.expression.substring(start, cur);
        final ConstPattern p = new ConstPattern(s);
        this.add(p);
    }
    
    private void addVariable(final int start, final int cur) throws PatternException {
        if (start >= cur) {
            return;
        }
        final String s = this.expression.substring(start, cur);
        final int p1 = s.indexOf("${");
        final int p2 = s.lastIndexOf(125);
        if (p1 >= 0 && p1 < p2) {
            final PlaceholderPattern p3 = new PlaceholderPattern();
            p3.parse(s);
            p3.setIsVariable(true);
            this.add(p3);
        }
        else {
            final VariablePattern p4 = new VariablePattern();
            p4.parse(s);
            this.add(p4);
        }
    }
    
    public void add(final IPattern pattern) {
        this.items.add(pattern);
    }
    
    public void add(final int index, final IPattern pattern) {
        this.items.add(index, pattern);
    }
    
    public void clear() {
        this.items.clear();
        this.expression = null;
        this.isVariable = false;
    }
    
    public IPattern get(final int index) {
        return this.items.get(index);
    }
    
    public String getExpression() {
        return this.toString();
    }
    
    public int getLength() {
        return 0;
    }
    
    public int getType() {
        return 1;
    }
    
    public int indexOf(final IPattern pattern) {
        return this.items.indexOf(pattern);
    }
    
    public void remove(final int index) {
        this.items.remove(index);
    }
    
    public void setExpression(final String value) {
        try {
            this.parse(value);
        }
        catch (final PatternException ex) {
            this.clear();
        }
    }
    
    public void setLength(final int value) {
    }
    
    public int size() {
        return this.items.size();
    }
    
    public int match(final String s, int start, final Map values) {
        if (this.isVariable) {
            return -1;
        }
        int cur = 0;
        int next = start;
        while (cur < this.size()) {
            switch (this.get(cur).getType()) {
                case 0: {
                    next = this.get(cur).match(s, start, values);
                    if (next == -1) {
                        return -1;
                    }
                    break;
                }
                case 1: {
                    if (this.get(cur).getLength() > 0) {
                        next = this.get(cur).match(s, start, values);
                        if (next == -1) {
                            return -1;
                        }
                        final String value = s.substring(start, next);
                        values.put(this.get(cur).getExpression(), value);
                        break;
                    }
                    else {
                        if (cur >= this.size() - 1) {
                            final String value = s.substring(start);
                            values.put(this.get(cur).getExpression(), value);
                            next = s.length();
                            break;
                        }
                        if (this.get(cur + 1).getType() == 0) {
                            next = s.indexOf(this.get(cur + 1).getExpression(), start + 1);
                            if (next == -1) {
                                return -1;
                            }
                            final String value = s.substring(start, next);
                            values.put(this.get(cur).getExpression(), value);
                            break;
                        }
                        else {
                            final int backLen = this.statNextLength(cur);
                            if (backLen == -1 || start >= s.length() - backLen) {
                                return -1;
                            }
                            next = s.length() - backLen;
                            final String value2 = s.substring(start, next);
                            values.put(this.get(cur).getExpression(), value2);
                            break;
                        }
                    }
                    break;
                }
            }
            start = next;
            ++cur;
        }
        return start;
    }
    
    private int statNextLength(final int cur) {
        int len = 0;
        for (int i = cur + 1; i < this.size(); ++i) {
            switch (this.get(i).getType()) {
                case 0: {
                    len += this.get(i).getLength();
                    break;
                }
                case 1: {
                    if (this.get(i).getLength() > 0) {
                        len += this.get(i).getLength();
                        break;
                    }
                    return -1;
                }
                default: {
                    return -1;
                }
            }
        }
        return len;
    }
    
    public String replace(final ITranslator translator) {
        final StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < this.size(); ++i) {
            buffer.append(this.get(i).replace(translator));
        }
        if (!this.isVariable) {
            return buffer.toString();
        }
        final String[] ret = translator.translate(buffer.toString(), 0);
        if (ret == null) {
            return buffer.toString();
        }
        return (ret.length == 0) ? "" : ret[0];
    }
}
